require "import"
import "android.app.*"
import "android.os.*"
import "android.widget.*"
import "android.view.*"
import "android.content.*"
import "android.provider.*"
import "android.media.*"
import "java.io.File"

activity.setTheme(android.R.style.Theme_DeviceDefault)

function getMediaFolders(mediaType)
    local folders = {}
    local folderPaths = {}
    
    local projection
    local uri
    
    if mediaType == "audio" then
        projection = {
            MediaStore.Audio.Media._ID,
            MediaStore.Audio.Media.DATA
        }
        uri = MediaStore.Audio.Media.EXTERNAL_CONTENT_URI
    else
        projection = {
            MediaStore.Video.Media._ID,
            MediaStore.Video.Media.DATA
        }
        uri = MediaStore.Video.Media.EXTERNAL_CONTENT_URI
    end
    
    local cursor = activity.getContentResolver().query(
        uri,
        projection,
        nil,
        nil,
        nil
    )
    
    if cursor then
        while cursor.moveToNext() do
            local path = cursor.getString(cursor.getColumnIndex(MediaStore.Audio.Media.DATA))
            if path then
                local folder = File(path).getParent()
                if folder and not folderPaths[folder] then
                    folderPaths[folder] = true
                    table.insert(folders, {
                        path = folder,
                        name = folder:match("([^/]+)$"),
                        type = mediaType
                    })
                end
            end
        end
        cursor.close()
    end
    
    return folders
end

function getMediaFilesInFolder(folderPath, mediaType)
    local files = {}
    
    local projection
    local uri
    local selection
    
    if mediaType == "audio" then
        projection = {
            MediaStore.Audio.Media._ID,
            MediaStore.Audio.Media.DISPLAY_NAME,
            MediaStore.Audio.Media.DATA
        }
        uri = MediaStore.Audio.Media.EXTERNAL_CONTENT_URI
        selection = MediaStore.Audio.Media.DATA .. " LIKE ?"
    else
        projection = {
            MediaStore.Video.Media._ID,
            MediaStore.Video.Media.DISPLAY_NAME,
            MediaStore.Video.Media.DATA
        }
        uri = MediaStore.Video.Media.EXTERNAL_CONTENT_URI
        selection = MediaStore.Video.Media.DATA .. " LIKE ?"
    end
    
    local cursor = activity.getContentResolver().query(
        uri,
        projection,
        selection,
        {folderPath .. "/%"},
        nil
    )
    
    if cursor then
        while cursor.moveToNext() do
            local name = cursor.getString(cursor.getColumnIndex(MediaStore.Audio.Media.DISPLAY_NAME))
            local path = cursor.getString(cursor.getColumnIndex(MediaStore.Audio.Media.DATA))
            if path and name then
                table.insert(files, {
                    name = name,
                    path = path,
                    type = mediaType
                })
            end
        end
        cursor.close()
    end
    
    return files
end

function deleteMediaFile(mediaFile)
    local file = File(mediaFile.path)
    if file.exists() then
        if file.delete() then
            if mediaFile.type == "audio" then
                activity.getContentResolver().delete(
                    MediaStore.Audio.Media.EXTERNAL_CONTENT_URI,
                    MediaStore.Audio.Media.DATA .. "=?",
                    {mediaFile.path}
                )
            else
                activity.getContentResolver().delete(
                    MediaStore.Video.Media.EXTERNAL_CONTENT_URI,
                    MediaStore.Video.Media.DATA .. "=?",
                    {mediaFile.path}
                )
            end
            return true
        end
    end
    return false
end

function showPlayerDialog(mediaFile)
    local dialog = Dialog(activity)
    dialog.setTitle(mediaFile.name)
    
    local layout = LinearLayout(activity)
    layout.setOrientation(1)
    layout.setPadding(20, 20, 20, 20)
    
    local videoView = nil
    local mediaPlayer = nil
    
    if mediaFile.type == "video" then
        videoView = VideoView(activity)
        local params = LinearLayout.LayoutParams(-1, 500)
        videoView.setLayoutParams(params)
        layout.addView(videoView)
        
        mediaPlayer = MediaPlayer()
        
        pcall(function()
            -- Initialize MediaPlayer first
            mediaPlayer.setDataSource(mediaFile.path)
            mediaPlayer.prepare()
            
            -- Then set up VideoView
            videoView.setVideoPath(mediaFile.path)
            videoView.setOnPreparedListener{
                onPrepared = function(mp)
                    -- Use the prepared MediaPlayer instance
                    if mp then
                        mediaPlayer = mp
                    end
                    videoView.start()
                end
            }
            
            videoView.setOnErrorListener{
                onError = function(mp, what, extra)
                    Toast.makeText(activity, "Error playing video. Please try again.", Toast.LENGTH_SHORT).show()
                    return true
                end
            }
        end)
    else
        mediaPlayer = MediaPlayer()
        pcall(function()
            mediaPlayer.setDataSource(mediaFile.path)
            mediaPlayer.prepare()
            mediaPlayer.start()
        end)
    end
    
    local settingsBtn = Button(activity)
    settingsBtn.setText("⚙️ Settings")
    layout.addView(settingsBtn)
    
    local controlsLayout = LinearLayout(activity)
    controlsLayout.setOrientation(0)
    controlsLayout.setGravity(Gravity.CENTER)
    
    local rewindBtn = Button(activity)
    rewindBtn.setText("⏪ -10s")
    
    local playBtn = Button(activity)
    playBtn.setText("⏸️")
    
    local forwardBtn = Button(activity)
    forwardBtn.setText("+10s ⏩")
    
    controlsLayout.addView(rewindBtn)
    controlsLayout.addView(playBtn)
    controlsLayout.addView(forwardBtn)
    layout.addView(controlsLayout)
    
    local currentSpeed = 1.0
    
    settingsBtn.onClick = function()
        local settingsDialog = Dialog(activity)
        settingsDialog.setTitle("Settings")
        
        local settingsLayout = LinearLayout(activity)
        settingsLayout.setOrientation(1)
        settingsLayout.setPadding(20, 20, 20, 20)
        
        local renameBtn = Button(activity)
        renameBtn.setText("Rename " .. mediaFile.type:gsub("^%l", string.upper))
        settingsLayout.addView(renameBtn)
        
        local speedBtn = Button(activity)
        speedBtn.setText("Playback Speed (" .. currentSpeed .. "x)")
        settingsLayout.addView(speedBtn)
        
        speedBtn.onClick = function()
            local speedDialog = AlertDialog.Builder(activity)
            speedDialog.setTitle("Select Playback Speed")
            
            local speeds = {"0.5x", "1.0x", "1.5x", "2.0x", "3.0x", "4.0x", "5.0x"}
            speedDialog.setItems(speeds, {
                onClick = function(dialog, which)
                    local newSpeed = tonumber(speeds[which + 1]:match("(%d+%.?%d*)x"))
                    currentSpeed = newSpeed
                    speedBtn.setText("Playback Speed (" .. currentSpeed .. "x)")
                    
                    if Build.VERSION.SDK_INT >= 23 and mediaPlayer then
                        pcall(function()
                            -- Store current position
                            local currentPosition = mediaPlayer.getCurrentPosition()
                            
                            -- Pause everything
                            if mediaFile.type == "video" then
                                videoView.pause()
                            end
                            mediaPlayer.pause()
                            
                            -- Update speed
                            local params = mediaPlayer.getPlaybackParams()
                            params.setSpeed(newSpeed)
                            mediaPlayer.setPlaybackParams(params)
                            
                            -- Restore position and resume
                            mediaPlayer.seekTo(currentPosition)
                            
                            if mediaFile.type == "video" then
                                videoView.seekTo(currentPosition)
                                videoView.start()
                            end
                            mediaPlayer.start()
                        end)
                    end
                end
            })
            
            speedDialog.show()
        end
        
        renameBtn.onClick = function()
            local input = EditText(activity)
            input.setText(mediaFile.name)
            
            AlertDialog.Builder(activity)
                .setTitle("Rename " .. mediaFile.type:gsub("^%l", string.upper))
                .setView(input)
                .setPositiveButton("Rename", {
                    onClick = function()
                        local newName = tostring(input.getText())
                        local oldFile = File(mediaFile.path)
                        local newFile = File(oldFile.getParent(), newName)
                        if oldFile.renameTo(newFile) then
                            mediaFile.name = newName
                            dialog.setTitle(newName)
                            Toast.makeText(activity, "Renamed successfully", Toast.LENGTH_SHORT).show()
                        end
                    end
                })
                .setNegativeButton("Cancel", nil)
                .show()
        end
        
        settingsDialog.setContentView(settingsLayout)
        settingsDialog.show()
    end
    
    playBtn.onClick = function()
        if mediaFile.type == "video" then
            if videoView.isPlaying() then
                videoView.pause()
                mediaPlayer.pause()
                playBtn.setText("▶️")
            else
                videoView.start()
                mediaPlayer.start()
                playBtn.setText("⏸️")
            end
        else
            if mediaPlayer.isPlaying() then
                mediaPlayer.pause()
                playBtn.setText("▶️")
            else
                mediaPlayer.start()
                playBtn.setText("⏸️")
            end
        end
    end
    
    forwardBtn.onClick = function()
        local currentPos = mediaPlayer.getCurrentPosition()
        local newPos = currentPos + 10000
        local duration = mediaPlayer.getDuration()
        
        if newPos > duration then
            newPos = duration
        end
        
        mediaPlayer.seekTo(newPos)
        if mediaFile.type == "video" then
            videoView.seekTo(newPos)
        end
    end
    
    rewindBtn.onClick = function()
        local currentPos = mediaPlayer.getCurrentPosition()
        local newPos = currentPos - 10000
        
        if newPos < 0 then
            newPos = 0
        end
        
        mediaPlayer.seekTo(newPos)
        if mediaFile.type == "video" then
            videoView.seekTo(newPos)
        end
    end
    
    dialog.setOnDismissListener{
        onDismiss = function()
            if mediaFile.type == "video" then
                videoView.stopPlayback()
            end
            if mediaPlayer then
                mediaPlayer.release()
            end
        end
    }
    
    dialog.setContentView(layout)
    dialog.show()
end

function onKeyDown(keyCode, event)
    if keyCode == KeyEvent.KEYCODE_BACK then
        if currentPath then
            showFolders(currentTab)
            return true
        end
    end
    return false
end

currentTab = "audio"
currentPath = nil

function onCreate(savedInstanceState)
    activity.setTitle("Media Player")
    
    if Build.VERSION.SDK_INT >= 23 then
        activity.requestPermissions({
            "android.permission.READ_EXTERNAL_STORAGE",
            "android.permission.WRITE_EXTERNAL_STORAGE"
        }, 1)
    end
    
    local layout = LinearLayout(activity)
    layout.setOrientation(1)
    layout.setPadding(16, 16, 16, 16)
    
    local tabLayout = LinearLayout(activity)
    tabLayout.setOrientation(0)
    
    local audioTab = Button(activity)
    audioTab.setText("Audio")
    local videoTab = Button(activity)
    videoTab.setText("Video")
    
    tabLayout.addView(audioTab)
    tabLayout.addView(videoTab)
    layout.addView(tabLayout)
    
    local listView = ListView(activity)
    local adapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1)
    local folders = {}
    local currentFiles = {}
    
    function showFolders(mediaType)
        currentTab = mediaType
        currentPath = nil
        adapter.clear()
        folders = getMediaFolders(mediaType)
        for _, folder in ipairs(folders) do
            adapter.add("📁 " .. folder.name)
        end
    end
    
    function showFiles(folderPath, mediaType)
        currentPath = folderPath
        adapter.clear()
        adapter.add("🔙 Back")
        currentFiles = getMediaFilesInFolder(folderPath, mediaType)
        for _, file in ipairs(currentFiles) do
            adapter.add(mediaType == "audio" and "🎵 " .. file.name or "🎬 " .. file.name)
        end
    end
    
    audioTab.onClick = function()
        showFolders("audio")
    end
    
    videoTab.onClick = function()
        showFolders("video")
    end
    
    listView.setAdapter(adapter)
    
    listView.onItemClick = function(parent, view, position, id)
        if currentPath == nil then
            local folder = folders[position + 1]
            if folder then
                showFiles(folder.path, folder.type)
            end
        else
            if position == 0 then
                showFolders(currentTab)
            else
                local file = currentFiles[position]
                if file then
                    showPlayerDialog(file)
                end
            end
        end
    end
    
    listView.onItemLongClick = function(parent, view, position, id)
        if currentPath and position > 0 then
            local file = currentFiles[position]
            if file then
                AlertDialog.Builder(activity)
                    .setTitle("Delete " .. file.type:gsub("^%l", string.upper))
                    .setMessage("Are you sure you want to delete '" .. file.name .. "'?")
                    .setPositiveButton("Delete", {
                        onClick = function()
                            if deleteMediaFile(file) then
                                Toast.makeText(activity, "File deleted successfully", Toast.LENGTH_SHORT).show()
                                showFiles(currentPath, currentTab)
                            else
                                Toast.makeText(activity, "Failed to delete file", Toast.LENGTH_SHORT).show()
                            end
                        end
                    })
                    .setNegativeButton("Cancel", nil)
                    .show()
                return true
            end
        end
        return false
    end
    
    layout.addView(listView)
    activity.setContentView(layout)
    
    showFolders("audio")
end